//layout toc title(subsubsection) link(subsubsection) subsubsubsecitalic secbold secbhline

//toctitle Stata level 2
//section Tips and tricks for working with Stata
//subsection Stata basics

//slide ------------------------------------------------------------------------
//title Passing commands to Stata

/*txt
{pstd} 
There are three ways of telling Stata what you want it to do

{pmore}
click on the menu. I do this very rarely, mainly to look for files or import files
from excel.

{pmore}
type in the command window. I do this a lot, but only for experimentation. 

{pmore}
write your commands in a .do file. This is the main way of doing things. It allows
me to keep track of what I am doing, and it keeps a paper trail of what I have done 
in case someone wants to replicat what I have done. 

{pstd}
A result can only appear in my article or presentation if it is the result of 
running a .do file

{pstd}
I do experiment in the command window, but the command I am happy with has to be 
coppied in the .do file.

{pstd}
I mainly work in Stata's do-file editor, which you can start by typing 
{stata "doedit"}
txt*/

//endslide --------------------------------------------------------------------- 


//slide ------------------------------------------------------------------------
//title Work flow

/*txt
{pstd}
Real research is too long for a single .do file. Instead you need to break this
up in several smaller do files.

{pstd}
Create one master .do file that executes these in turn; a .do file can execute 
another .do file (say {cmd:cceast_dta01.do}, by including the command 
{cmd:do cceast_dta01.do}

{pstd}
Have a naming system for your .do files. I typically have a small abbreviation of
a project (e.g. cceast), and add to that either _dta or _ana for data preparation
or analysis files. After that I add a number. 

{pstd}
Numbering the files prevents names like "final", "really_final", "no_seriously_I_am_done", etc.

{pstd}
I have at least two directories: "working" and "posted". I can change anything in
the working directory, but once something is in posted I cannot change it anymore. 

{pstd}
I will put files in posted when I present my results at a conference or submit a
paper to a journal. This ensures I can always replicate those results.
txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Navigation

/*txt 
{pstd}
Stata has a working directory and system directories.

{pstd}
The working directory is where Stata will look for (data and .do) files when you 
don't specify the directory. 

{pstd}
If you type in Stata {stata "pwd"} (print working directory) it will tell you where
Stata is. 

{pstd}
You can change the working directory using {cmd cd}

{pstd}
You can also use relative paths. Say you are in the working directory, and your
data is stored in posted/data, you can type {cmd:use ../posted/data/datafile.dta}.

{pstd}
This is useful when you work on different computers. If you have to set the 
directory only once, in the master .do file, and all other .do files will only use
relative paths, then you have to change the {cmd:cd} command only once in order
to make it work on your other computers. 

{pstd}
system directories is where Stata looks for its programs, help-files, etc.

{pstd}
You can find out where that is by typing {stata "sysdir"}
txt*/
//endslide ---------------------------------------------------------------------
 

//slide ------------------------------------------------------------------------
//title Customize your Stata

/*txt
{pstd}
You can set the scheme of your output window using edit --> preferences --> 
general preferences

{pstd}
You can set the font by right clicking on a window and choose font... I like 
lucida console 

{pstd}
You can let Stata execute a couple of commands every time it starts up. By creating
a .do file called {cmd:profile.do} and store it in your PERSONAL folder 
(see: {cmd:sysdir}).

{pstd}
My profile.do reads:

{cmd}
        noi di as txt _n"Current projects:"

        noi di as txt "F4" as result " SS18 Stata_L2"
        global F4 cd "D:\Mijn documenten\onderwijs\konstanz\ss18\stata_l2\";

        exit
{txt}

{pstd}
This makes that if I press F4 Stata will {cmd:cd} to the directory of this course.
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------ 
//title information available in a Stata dataset
/*txt
{pstd}
The main place where we store and access information is the dataset, and 
specifically the values a variable takes.
txt*/

//ex
sysuse auto, clear
browse
list foreign rep78 in 1/10, nolabel
display rep78
display rep78[5]
//endex

/*txt
{pstd}
The dataset can contain additional information in the form of variable, value, 
and data labels
txt*/

//ex
desc
fre rep78
//endex

/*txt
{pstd}
{cmd:desc} also returns the type of the variable, this gives information about 
whether a variable a string or a numeric variable, and if it is a numeric variable
about the range of possible values.

{pstd}
In addition {cmd:desc} also provides information about the format of a variable,
that is, how the values are supposed to be displayed. This can be particularly 
useful for finding time variables.
 
{pstd}
We can add additional comments to variables and datasets in addition to the labels
using {help notes}. A more general version of notes are {help char: characteristics}.
These are used to store information in the dataset when {help xtset} or {help stset} 
the data.
txt*/
//endslide ---------------------------------------------------------------------

//slide
//title Additional information available in a stata session

/*txt
{pstd}
You can store information in macros

{pstd}
Stata also has the possibility to store information in scalars or matrices.

{pstd}
Many commands return results, which can be accessed until the next command is run.
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title returned results

/*txt
{pstd}
Many command leave the results behind in memory. These are returned results.
txt*/

//ex
sum mpg
return list
di r(mean)

sum mpg, meanonly
return list
//endex

/*txt
{pstd}
Because each command can return results, you should not expect these to persist 
for long. If you need them store the desired results immediately after the command,
e.g. in a local macro

{pstd}
There are various types of returned results:

{pmore}
returned results ({cmd:r(}{it:something}{cmd:)}, and {cmd:return list}), these 
come from general, non-estimation commands.

{pmore}
ereturned results ({cmd:e(}{it:something}{cmd:)} and {cmd:ereturn list}), these
come from estimation commands, like {cmd:regress}

{pmore}
sreturned results ({cmd:s(}{it:something}{cmd:)} and {cmd:sreturn list}), these
come from subprograms.

{pmore}
creturned results ({cmd:c(}{it:something}{cmd:)}) contains the value of system 
parameters and settings, along with certain constants such as the value of pi.A 
full list can be found at {help creturn: help creturn}.
txt*/

//endslide ---------------------------------------------------------------------


//subsection macros

//slide ------------------------------------------------------------------------
//title Macro
/*txt
{pstd}
A macro is a shorthand, it is one thing standing for another
txt*/

//ex
sysuse auto, clear
local xvars = "mpg foreign"
reg price `xvars'
reg price mpg foreign
//endex

/*txt
{pstd}
In the second line of this example we defined a local macro called xvars, which
stands for / contains the string "mpg foreign"

{pstd} 
The syntax is 

{pmore}
{cmd:local} {it:macroname} {it:content}

{pstd}
If we later want to refer to the contents of the local macro xvars we type 
`xvars', that is the {it:macroname} with left and right single quotes.

{pstd}
When Stata sees a line it will first look for macros and replace that with its 
content

{pstd}
So when Stata saw line 3, the first thing it did is look up what was in the 
local macro called xvars, and replaced `xvars' with its contents, and only than 
did it try to execute the command. 

{pstd}
So the third and fourth line are equivalent as far as Stata is concerned.
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title what is in a macro?

/*txt
{pstd}
It can contain anything, but is always a string
txt*/

//ex
local mac "foo"
di `mac'
//endex

/*txt
{pstd}
I said that the content of a macro is always a string, then why did this return 
an error?

{pstd}
The double quotes are there to indicate that this is a string, but they are not 
part of the string.

{pstd}
So `mac' contains {it:foo} not {it:"foo"}

{pstd}
So for the second line Stata saw {cmd:di foo}, and since there were no double 
quotes Stata assumed we wanted to look at a variable (or scalar) foo, could not
find that variable and returned the error message.

{pstd}
This will work:
txt*/

//ex
local mac "foo"
di "`mac'"
//endex

//endslide

//slide ------------------------------------------------------------------------
//title Macros containing numbers
/*txt
{pstd}
Because the quotes are stripped, macros may also contain numbers.

{pstd}
They are stored as strings, but as soon as Stata replaces the name of the macro
with its content it will see them as numbers, as there are no quotes around them.
txt*/

//ex
local mac 1
di `mac'
//endex

/*txt
{pstd}
However, numbers stored in macros are not quite as precise as numbers stored in
scalars.

{pstd}
Scalars are stored in double precision (15-16 decimal digits), while locals have
about 12 decimal digits, sometimes more, but never less than 11.
txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Scalars

/*txt
{pstd}
A scalar is a "container" containing one element, either a string or a number.

{pstd}
It is good practice to use tempnames for scalars as they share the same namespace
as variables

{pstd}
Unlike a macro the name is not immediately replaced by its contents
txt*/

//ex
sum mpg, meanonly
tempname m_mpg
scalar `m_mpg' = r(mean)
scatter mpg price, yline(`m_mpg')
//endex

/*txt
{pstd}
In order to replace the scalar name with its contents you can type {cmd:`=}{it:scalarname}{cmd:'}
txt*/

//ex
sum mpg, meanonly
tempname m_mpg
scalar `m_mpg' = r(mean)
scatter mpg price, yline(`=`m_mpg'')
//endex

//graph Graph
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title compound quotes

/*txt
{pstd}
We can make the double quotes part of the string by surrounding them with 
compound quotes
txt*/

//ex
local mac `" "foo" "'
di `mac'
di `"`mac'"'
di `"|`mac'|"'
di "`mac'"
//endex

/*txt
{pstd}
The content of the macro `mac' is in this example {it:<space>}"foo"{it:<space>}.

{pstd}
So in the first {cmd:di}, Stata sees: {cmd:di {it:<space>}{it:<space>}"foo"{it:<space>}}

{pstd}
In the second {cmd:di}, Stata sees: {cmd:{it:<space>}`"{it:<space>}"foo"{it:<space>}"'}

{pstd}
These spaces are more visible by surounding them with pipes: now Stata sees 
{cmd:{it:<space>}`"|{it:<space>}"foo"{it:<space>}|"'}

{pstd}
The final one gets into trouble because Stata does not know that the outer double 
quotes should "wrap around" the quotes in the macro. 
txt*/

//endslide --------------------------------------------------------------------- 

//slide ------------------------------------------------------------------------
//title = or no =

/*txt
{pstd}
If the content of a macro is a string, and quotes are stripped anyhow, then the 
quotes when defining the macro seem redundant
txt*/

//ex
local mac foo
di "`mac'"
//endex

/*txt
{pstd}
However, that is only true if we did not include a "="

{pstd}
Including an "=" means that what comes after the equal sign is an expression, and
the result of evaluating that expression is to be the content of the macro.

{pstd}
So when Stata sees {cmd:local mac = foo}, then Stata starts looking for the 
variable or scalar foo, which it can than put into the local macro `mac'. It 
cannot find it, and will return an error message
txt*/

//ex
local mac = foo
//endex
= or no =
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title local versus global

/*txt
{pstd}
We have thus far used local macros.

{pstd}
In a .do file they exist as long as that .do file is running, but disappear 
immediately afterwards.

{pstd}
So if you run a line of your do file which defines the local macro and than 
run another line that uses that local macro, that local macro will no longer exist.

{pstd}
That sounds awkward, but it is actually extremely useful. 

{pstd}
The alternative is a global macro: a macro that persists after you are done running
a .do file.

{pstd}
In a datapreparation or data analysis phase you can easily work hours on end, 
have (lunch) breaks in between etc. What happens when you use global macros? You
defined them early in the morning and you may or may not have changed somewhere 
along the way. You can easily imagine a situation where you think your global 
contains one thing, but actually contains something else.

{pstd}
So global macros are dangerous, and generally considered bad practice.
txt*/

//endslide ---------------------------------------------------------------------


//slide ------------------------------------------------------------------------
//title tempname tempfile tempvar

/*txt
{pstd}
{cmd:tempname}, {cmd:tempfile}, and {cmd:tempvar} are special types of local 
macros.

{pmore}
{cmd:tempfile} is a local macro containing a name for a matrix or scalar that is 
guaranteed not to exist, and that will be removed at the end of the session.

{pmore}
{cmd:tempvar} is a local macro containing a filename that is guaranteed not to 
exist, and that will be removed at the end of the session.

{pmore}
{cmd:tempname} is a local macro containing a variable name that is guaranteed 
not to exist, and that will be removed at the end of the session.

{pstd}
These are good ways of storing intermediate results that you need to store for
a very short time.

{pstd}
For example, I often use {cmd:tempfile} in combination with {cmd:merge}. 

{pmore}
I prepare a dataset for merging, and store it as a tempfile

{pmore}
I open the other dataset, and merge the tempfile in.
txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Passing local macros to another .do file

/*txt
{pstd} 
You can run a .do file called foo.do by typing {cmd: do foo.do}

{pstd}
You can also run that .do file by typing {cmd: do foo.do something else}

{pstd}
This will make the following local macros available at the beginning of foo.do:

{pmore}
`0' containing: something else

{pmore}
`1' containing: something

{pmore}
`2' containing: else

{pstd}
foo.do could do some complicated/fiddly manipulations of a variable

{pstd}
You want to apply those to multiple variable, say {cmd:var1} and {cmd:var2}

{pstd}
Now you can create another .do file that contains the lines:

{cmd}
        do foo.do var1
        do foo.do var2
{txt}

{pstd}
If you find an error in foo.do, you only have to fix it once, saving you a lot 
of potential to include inconsistencies and bugs.
txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Leaving local macros behind

/*txt
{pstd}
Say you have a .do file bar.do that calls a .do file foo.do

{pstd}
foo.do makes local macros that you want bar.do to have access to

{pstd}
Instead of the line {cmd: do foo.do} you can add the line {cmd:include foo.do} in 
bar.do

{pstd}
This will run foo.do as if it was actually part of bar.do. That way any local 
macros defined in foo.do will be available in bar.do

{pstd}
This is a good way to store settings that will be used in multiple .do files

{pstd}
A (dangerous) alternative is /*digr*/ .
txt*/
//include
//endslide ---------------------------------------------------------------------

//digr -------------------------------------------------------------------------
//title c_local
//label c_local

/*txt
{pstd}
With {cmd:c_local} you can create a local macro one level below. 

{pstd}
So if bar.do calls foo.do, and foo.do contains the line {cmd:c_local blup 1} then 
that will create a local macro in bar.do called `blup' containing 1

{pstd}
If you are later reading bar.do you have no indication that foo.do will create that
local macro. So it can be very hard to later figure out where that local macro
`blup' came from. This is why I labeled it as dangerous.

{pstd}
This is why it is not documented, and not even {help undocumented}.

{pstd}
To quote Nick Cox: {browse "https://www.stata.com/statalist/archive/2005-11/msg00405.html"}:

{pmore}
-c_local- is not 
documented; it is not even "undocumented" 
(-help undocumented-). So, how does anyone 
outside StataCorp know about it? 

{pmore}
What happens is this: after a long period
of Stata use in which you have done well, 
Stata will speak to you: 

{pmore}
"Greetings! You have reached the seventh 
level of Stata, and I name you Statafriend. 

{pmore}
You will now be initiated into seven
Stata secrets. The first is -c_local-." 

{pmore}
and so forth, but the rest of it is
probably not of interest.

{pstd}
Since this is only a second level Stata course you still have a way to go...
txt*/

//enddigr ----------------------------------------------------------------------


//slide ------------------------------------------------------------------------
//title extended macro functions

/*txt
{pstd}
{help extended_fcn:extended macro functions} are function you can use when 
defining a macro 
txt*/

//ex
local varlab : var label mpg
di "`varlab'"
//endex

/*txt
{pstd}
You can also use that on the fly
txt*/

//ex
di "`: var label price'"
//endex

/*txt
{pstd}
The most useful are functions for

{pmore}
extracting data attributes (labels, characteristics, variable types)

{pmore}
file names and file paths

{pmore}
formating results

{pmore}
{help macrolists:manipulating lists}

{pmore}
matrices

{pmore}
parsing strings
txt*/
//endslide ---------------------------------------------------------------------

//subsection looping

//slide ------------------------------------------------------------------------
//title An example loop

//ex
forvalues i = 1/10 {
    di `i'
}
//endex

/*txt
{pstd}
{cmd:forvalues} tells Stata that we want to loop

{pstd}
the {cmd:i} after forvalues is the name of a local macro that will exist when 
the loop runs

{pstd}
{cmd:= 1/10} tells Stata the values the local i should take: 1 for the first time,
2 for the second time, ..., 10 for the tenth time, and then it stops.

{pstd}
{cmd:{c -(}} and {cmd:{c )-}}: whatever is between these braces is going to be 
repeated 

{pstd}
{cmd:di `i'} displays the content of the local macro i. So the first time it evaluates
to {cmd: di 1}, the second time to {cmd:di 2}, etc.

{pstd}
previously we talked about a .do file foo.do that can be used to manipulate var1 
and var2. What if we have var1 till var50?

{cmd}
        forvalues i = 1/50 {
            do foo.do var`i'
        }
{txt}

{pstd}
before Stata executes a line it first replaces macros with their content.

{pstd}
So the first time around it sees {cmd:do foo.do var1}, the second time around
it sees {cmd:do foo.do var2}, etc.

{pstd}
In this case it is necessary that there is no space between {cmd:var} and {cmd:`i'}
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title different types of loops

/*txt
{pstd}
{cmd:forvalues i = 2(2)100 }

{pmore}
We loop over the values 2, 4, 6, ..., 100

{pmore}
This is the type of loop I use most

{pstd}
{cmd:foreach var of varlist * }

{pmore}
We loop over all variables in the dataset

{pmore}
This is useful for specific list like varlists, numlist, or locals

{pstd}
{cmd:while `diff' > 1e-6 }

{pmore}
`diff' is probably a tempname for a scalar, and we continue the loop till this 
scalar is less than 1e-6 (0.0000001)

{pmore}
This is usful for when you want to iteratively optimise something. Often there 
are beter suits of commands for that, e.g. {help ml}, {help nl}, {help gmm}, so
that is very rarely used.
txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Try it yourself

/*txt
{pstd}
The variables weight length and turn all measure the size of the car. Maybe we 
want to combine those variables into one. In order to do so we need to make sure
the are measured in the same unit. 

One possiblity would be the percentile score; the proportion of cars that is 
smaller. Here is how I would do this for weight
txt*/

//ex
egen i = rank(weight)
count if !missing(weight)
gen p_weight = (i - .5)/r(N)
//endex

/*txt
{pstd}
Use a loop to create percentile scores for weight, length, and turn.
txt*/


//endslide ---------------------------------------------------------------------

//subsection prefixes

//slide ------------------------------------------------------------------------
//title by

/*txt
{pstd}
Say we want a new variable containing the mean price for every level of repair 
status

{pstd}
We could do this with a loop
txt*/

//ex
levelsof rep78
local levs = r(levels)
gen mprice = .
foreach lev of local levs {
    sum price if rep78 == `lev', meanonly
    replace mprice = r(mean) if rep78 == `lev'
}
//endex

/*txt
{pstd}
Alternatively we could use the {cmd:by} prefix
txt*/

//ex
bysort rep78 : egen mprice2 = mean(price)
//endex

/*txt
{pstd}
We could use this to find the highest price within each level of repair status
txt*/

//ex
bys rep78 (price) : gen maxprice = price[_N]
//endex

/*txt
{pstd}
What would happen if price contained missing values?
txt*/

//ex
gen misprice = missing(price)
bys rep78 misprice (price): gen maxprice2 = price[_N] if misprice == 0
//endex


/*txt
{pstd}
Can you find the lowest price in each level of rep78?

Can you find the second highest price in each level of rep78?
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title statsby

/*txt
{pstd}
The {cmd:statsby} prefix allows you to execute a command for each level of a 
variable and store returned statistics in a dataset.

{pstd}
This is often useful for creating graphs of summary statistics
txt*/
//ex
sysuse nlsw88, clear
statsby m=r(mean) min=r(min) max=r(max) p75=r(p75) p25=r(p25) , ///
     by(industry) clear: sum wage, d
list
//endex
//endslide ---------------------------------------------------------------------

//subsection finding and manipulating groups of variables

//slide ------------------------------------------------------------------------
//title finding variables

/*txt
{pstd}
Most datasets contain a great many variables. Finding the variables you are looking
for can be a challenge. 

{pstd}
The {help lookfor} command can be useful, it allows you to search for variables
with a specific string in their name or labels
txt*/

//ex
sysuse auto, clear
lookfor repair
return list
//endex

/*txt
{pstd}
Say we want a list of all numeric variables

{pstd}
This is something {help ds} can do
txt*/

//ex
ds, has(type numeric)
//endex
//endslide ---------------------------------------------------------------------

//subsection numerical precision

//slide ------------------------------------------------------------------------
//title binary versus decimal

/*txt
{pstd}
If a computer stored numbers in decimal format we would not be surprised that we
could not store the number 1/3 exactly; we would have to stop storing 3s 
otherwise we would need an infinite amount of memory to store one number.

{pstd}
A computer however stores numbers in binary format. In binary some numbers we would
not consider problematic, are actualy like 1/3. The most common example is 0.1. 

{pstd}
So a lot of numbers we think are perfectly "normal" are in a computer actually 
rounded versions of that number.
txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title how a number is stored

/*txt
{pstd}
So how are numbers stored? 

{pstd}
We could say that we store a number up to 6 digits 
after the decimal point (ignoring that they  are actually stored in binary)?

{pstd}
This is problematic

{pmore}
We would store the number 1,000,000 with 13 significant digits

{pmore}
While we would store the number 0.0001 with only 3 significant digits

{pstd}
Instead a number is stored in three parts: the sign and two numbers, lets call
them a and b

{pstd}
If we would store the number in decimal format the number stored would then be
sign * a * 10^b

{pstd}
So if we decided on 6 significant digits we would store the number 1,000,000 as
+1*1,00000*10^6 and the number 0.0001 as +1*1.00000*10^-4

{pstd}
In real computers both a and b are binary numbers and we don't use 10^b, but 2^b
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title rounding errors: adding and subtracting

/*txt
{pstd}
This way of storing number allows us to reliably store number within a very large
range.

{pstd}
It has some quirks, for example adding numbers that differ by a large order of
magnitude can lead to quite large rounding errors.

{pstd}
We want to add 1,000,000 and 0.0001

{pmore}
Then we are adding +1*1,00000*10^6 and +1*1.00000*10^-4

{pmore}
In order to add them we would need to the exponent the same:

{pmore}
We would change +1*1.00000*10^-4 to +1*0.00000000001*10^6

{pmore}
However, we only stored 6 digits, so 0.00000000001 gets rounded to 0 
txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title storage types

/*txt
{pstd}
The problem with precision mainly occurs with fractional numbers, integers can be
stored exactly as long as they are not too big

{pstd}
There are three storage types for integers that differ with respect to the range
of numbers they can store and the amount of memory needed to store them

{pmore}
{cmd:byte}, which can store numbers between -127 and 100 and uses only one byte
per number

{pmore} 
{cmd:int} , which can store numbers between -32,767 and 32,740 and uses two bytes
per number

{pmore}
{cmd:long} which can store numbers between -2,147,483,647 and  2,147,483,620 and 
uses four bytes per number

{pstd}
There are two storage types for fractional numbers

{pmore}
{cmd:float} has a precision of about 8 decimal digits and takes four bytes per 
number

{pmore}
{cmd:double} has a precision of about 16 decimal digits and takes 8 bytes per 
number. A double can also store the largest range of numbers: between -8.988*10^307 
and 8.988*10^307.

{pstd}
The default storage type for variables is float. 

{pstd}
This is fine for storing data. We typically don't think we measured our variables
upto 8 digits accurate.

{pmore}
Say we ask someone's income. A respondent does not have her or his income exactly
in memory, he or she will round when answering that question. We can probably trust
the first two, maybe three, digits. So storing that with about 8 digits accuracy
is more than enough.

{pstd}
A float is probably not optimal for storing intermediate results of computations.
We want to minimise the rounding errors that happen at each step, and if we store
them as floats they can quickly add up. So for intermediate results in computations
you are better of using doubles. 
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title possible problems

/*txt
{pstd}
logical statements with fractional numbers

{pmore}
Say we want to summarize a variable x for only the observation for which var == 0.1

{pmore}
We type {cmd:sum x if var == 0.1} and Stata will tell you that no observations 
meet that criterium, while if you look at the data you see several such observations.

{pmore}
var is probably stored as a float, but Stata does computations in double precision

{pmore}
So Stata is comparing the float(0.1) from var to the double(0.1) in the expression
and finds that they are not equal.

{pmore}
The solution is to really don't do equality checks on fractional numbers.

{pstd}
Storing larger numbers that need to be stored exactly 

{pmore}
This is very common for ID variables. Say the first two digits stand for the 
country, the next two digits for the privince, the next three digits for the 
city, the next 4 digits for the household, the next two digits for the person, 
and the next two digits for the wave.

{pmore} 
Now we have 15 digits, and a float cannot store that. A double can.

{pmore}
If we have even larger numbers we can store them as strings.
txt*/

//endslide ---------------------------------------------------------------------

//subsection programming

//slide ------------------------------------------------------------------------
//title defining a program in a .do file

//ex
program drop _all
program define hello
    di "hello world"
end
hello
//endex

/*txt
{pstd}
A program starts with {cmd:program} and ends with {cmd:end} 

{pstd}
Whatever is in between is anything that you could also include in a regular .do 
file.

{pstd}
So now you can write Stata programs.
txt*/

//endslide --------------------------------------------------------------------- 

//slide ------------------------------------------------------------------------
//title Why would you want to do that?

/*txt
{pstd}
The most common application would be to use it to bootstrap some statistic

{pstd}
Occationally I do that to automate some extremely fiddly tasks that I need to do
repeatedly

{pstd}
Consider the following example
txt*/

//ex
sysuse nlsw88,clear 
reg wage c.ttl_exp##c.ttl_exp grade
nlcom -_b[ttl_exp]/(2*_b[c.ttl_exp#c.ttl_exp])
//endex

/*txt
{pstd}
So the maximum wage is obtained after 71 years of experience with a huge 
confidence interval

{pstd}
We may not trust that confidence interval as {cmd:nlcom} assumes that the sampling
distribution is normally distributed, and our statistic is something divided by a
number that could easily be 0. So we an expect strange things to happen, and to
be sure we would like to bootstrap this.
txt*/

//ex
program drop _all
sysuse nlsw88
program define toboot, rclass
    version 14
    syntax [if]
    marksample touse
    reg wage c.ttl_exp##c.ttl_exp grade if `touse'
    return scalar max = -_b[ttl_exp]/(2*_b[c.ttl_exp#c.ttl_exp])
end
toboot
return list

bootstrap max=r(max), reps(100) bca : toboot
estat bootstrap, bca
//endex
//endslide


//section Application: writing your own program for creating a codebook in .html

//slide ------------------------------------------------------------------------
//title Goal

/*txt
{pstd}
We will write a program {cmd:htmlcodebook} that creates a codebook in html format
for a specified Stata dataset.

{pstd}
The codebook will contain:

{pmore}
The name of the file with data label and notes

{pmore}
A list of variables with label and notes

{pmore}
A desription of each variables: a table if there are value labels or the number
of distinct values is small, summary statistics for unlabeled numeric variables 
with more values, and some examples for string variables.
txt*/

//ex
htmlcodebook using arc06.dta, saving(test.html) replace
//endex

/*txt
{pstd}
Such a program is not written in one go, but this happens in a large series of 
small steps.

{pstd}
I have created a large number of small exercises that simulate that process in a 
guided way.

{pstd}
These exercises will cover a large part material discussed before.
txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Starting

//codefile htmlcodebook00.do
/*txt
{pstd}
//ho_ignore We can see the first step by typing {stata "doedit htmlcodebook00.do"}

{pstd}
We start working in a .do file, and we start small. A .html file with only one 
line: "title".

{pstd}
It shows us what a program looks like, and how to write a file with Stata using 
the {help file} command.

{pstd}
It is important that we start our .do file with {cmd:program drop _all}. We are definig
a program, and Stata will return an error message if it already exists in memory.

{pstd}
The {help version} command makes sure that this program will continue to work in
future versions of Stata.

{pstd}
We specify the possible options with the {help syntax} command.

{pstd}
With the {help file} command we can write a file.
 
{pmore}
The first line says what kind of html file this is

{pmore}
The second and last line tells when the html file begins and ends.

{pmore}
The third and second last line tells when the body of the html file begins and
ends.

{pmore}
The fourth line appears is displayed in the html file: it is "title" displayed 
as a heading 1.

{pstd}
After we created the html file we let Stata display a link that will open it in
a browser.

{pstd}
We can see the file in plain text with the {help type} command.
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Add some lines to our .html file to make it prettier

/*txt
{pstd}
We can make the html file look prettier by including the following lines after 
{cmd:<html>} and before {cmd:<body>}.

{cmd}
    <style>
    body {
        width: 650px;
        margin: auto;
    }
    </style>
{txt}

{pstd}
It limits the width of the output and positions it in the middle of the screen.

{pstd}
Change the program to make it include those lines at the appropriate place.

//ho_ignore {pstd}{stata "doedit htmlcodebook01.do":solution}{p_end}
txt*/
//apcodefile htmlcodebook01.do

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title add a replace option

/*txt
{pstd}
If we run our .do file multiple times, than the file {cmd:test.html} is created
multiple times.

{pstd}
Stata never overwrites something unless you explicitly say so, i.e. specify the
{cmd: replace} option in the {cmd:file} command.

{pstd}
We want to copy that behavior: i.e. 

{pmore}
add a {cmd:replace} option to our {cmd:htmlcodebook} command, and 

{pmore}
only specify the {cmd:replace} option in {cmd:file}, when the user specified the 
{cmd:replace} option in {cmd:htmlcodebook}.

{pstd}
Look in {help syntax:help syntax} to find out how to implement on/off options 
(the user specified or did not specify the {cmd:replace} option).

//ho_ignore {pstd}{stata "doedit htmlcodebook02.do":solution}{p_end}
txt*/
//apcodefile htmlcodebook02.do
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Let the user specify a meaningful title

/*txt
{pstd}
"Title" is not a very meaningful title, we should add an option that allows
the user to specify its own title

{pstd}
Such an option would ask for a string, see {help syntax: help syntax} on how to
implement such an option

//ho_ignore {pstd}{stata "doedit htmlcodebook03.do":solution}{p_end}
txt*/
//apcodefile htmlcodebook03.do
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title What should the title be when the title() option is not specified?

/*txt
{pstd}
If the user did not specify the {cmd:title} option then we need to make a 
sensible choice. I suggest "Codebook for" and than the file the user specified 
in {cmd:using}

{pstd}
See {help ifcmd : help ifcmd} on how to implement different things depending on
whether or not an option was specified.

//ho_ignore {pstd}{stata "doedit htmlcodebook04.do":solution}{p_end}
txt*/
//apcodefile htmlcodebook04.do
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Add a list of variable names to our codefile

//codefile htmlcodebook05.do
/*txt
//ho_ignore {pstd}Type {stata "doedit htmlcodebook05"} to see the code{p_end}

{pstd}
At the very least a codebook should contain a list of variable names

{pstd}
In the .do file we can see how we can create a list in html using the {cmd:<ul>}
, {cmd:</ul>}, {cmd:<li>}, and {cmd:</li>} tags.
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Adding varialble labels to our variable list

/*txt
{pstd}
Change this program such that 

{pmore}
if there is a variable label then the variable list shows variable name : variable
lable

{pmore}
otherwise just the variable name

{pstd}
Use extended macro functions find the variable label 
(see {help extended_fcn: help extended_fcn})

{pstd}
Notice that that returns the variable name if no variable label is attached to 
that variable

//ho_ignore {pstd}{stata "doedit htmlcodebook06.do":solution}{p_end}
txt*/
//apcodefile htmlcodebook06.do
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Add data label to codebook

/*txt
{pstd}
Use extended macro functions to find the label belonging to the dataset.

{pstd}
If such a label exists, add it to the codebook underneath the title with 
{cmd:<h2>} and {cmd:</h2>} tags.

//ho_ignore {pstd}{stata "doedit htmlcodebook07.do":solution}{p_end}
txt*/
//apcodefile htmlcodebook07.do
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Add data notes to codebook

/*txt
{pstd}
Notes are stored as {help char:characteristics}, which can be accessed
using the {cmd:`: char'} extended macro function. 

{pstd}
The notes are named _dta[note1], _dta[note2], etc.

{pstd}
How many notes there are is stored in _dta[note0]

{pmore}
This is empty ("") if no notes were specified

{pmore}
Or contains the number of notes, when one or more notes exist

{pstd}
Add a list of data notes underneath the data label if such notes exist

//ho_ignore {pstd}{stata "doedit htmlcodebook08.do":solution}{p_end}
txt*/
//apcodefile htmlcodebook08.do
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Splitting the program up in smaller subroutines

//codefile htmlcodebook09.do
/*txt
//ho_ignore {pstd}To see the program type {stata "doedit htmlcodebook09.do"}{p_end}

{pstd}
Our program becomes bigger and bigger. To keep an overview, and make it easier
to spot errors and maintain the program it is best to split the program up into
different subroutines, each for a specific task.

{pstd}
Here we create a subroutine for describing a file

{pstd}
StataCorp often starts subroutines with capital leters, and I have copied that
convention.

{pstd}
Notice that the handle for our codebook, `book', was a {help tempname}, so local
to the program that created it. 

{pstd}
To solve this we define the handle in the main program. 

{pstd}
This means that it exists as long as the main program is being executed, even if
it call another program, like our subroutine.

{pstd}
That way we can pass that temporary name on to sub-routines as options

{pstd}
Use this trick to create another sub-routine for describing the variables, call 
it {cmd:Descvars}

//ho_ignore {pstd}{stata "doedit htmlcodebook10.do":solution}{p_end}
txt*/
//apcodefile htmlcodebook10.do
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Add variable notes to the variable list

/*txt
{pstd}
Change {cmd:Descvars} such that notes belonging to that variable are displayed
if they exist.

//ho_ignore {pstd}{stata "doedit htmlcodebook10.do":solution}{p_end}
txt*/
//apcodefile htmlcodebook11.do
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Add a list of value labels to our codebook

//codefile htmlcodebook12.do
/*txt
//ho_ignore {pstd}See the code in {stata "doedit htmlcodebook12.do"}{p_end}

{pstd}
Here we add for each variable a list of value labels to our codebook if value
labels exist

{pstd}
It uses the {help uselabel} command, which stores value labels as a new dataset

txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Add frequencies to our labels

//codefile htmlcodebook13.do
/*txt
//ho_ignore {pstd}See the code in {stata "doedit htmlcodebook13.do"}{p_end}

{pstd}
{help contract} changes the data in one observation per value of `var' and a new
variable `freq' that contains the number of observations with that value

{pstd}
We merge that with the file created by {cmd:uselabel}
txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Numeric variables without value labels

//codefile htmlcodebook14.do
/*txt
//ho_ignore {pstd}See the code in {stata "doedit htmlcodebook14.do"}{p_end}

{pstd}
For numeric variables with less than 10 values we tabulate the data

{pstd}
For more we start showing summary statistics.
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title display format for summary statistics

/*txt
{pstd}
I am not quite happy with the way the minimum is displayed.

{pstd}
Can't we copy the display format for that variable?

{pstd}
The display format for a variable can be extracted using extended macro functions

{pstd}
You can apply that format using the `: display ' extended macro function

{pstd}
Change the program to achieve this goal

//ho_ignore {pstd}{stata "doedit htmlcodebook15.do":solution}{p_end}
txt*/
//apcodefile htmlcodebook15.do

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Add other summary statistics

/*txt
{pstd}
Expand our program to also incude the 25th, 50th, 75th percentiles and the maximum.

//ho_ignore {pstd}{stata "doedit htmlcodebook16.do":solution}{p_end}
txt*/
//apcodefile htmlcodebook16.do
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title String variables

/*txt
{pstd}
We also want to show something for string variables.

{pstd}
Lets show the first 10 distinct values as an example

{pstd}
Change the program to achive that goal

//ho_ignore {pstd}{stata "doedit htmlcodebook17.do":solution}{p_end}
txt*/
//apcodefile htmlcodebook17.do
//endslide ---------------------------------------------------------------------


//slide ------------------------------------------------------------------------
//title Add links between the variable list and the value label list

//codefile htmlcodebook18.do
/*txt
//ho_ignore {pstd}See the code in {stata "doedit htmlcodebook18.do"}{p_end}

{pstd}
{cmd:Descvalues} creates markers, and in {cmd:Descvars} it adds links to those
markers

{pstd}
That way users can now jump from the list of variables to the list of value labels.

txt*/

//codefile htmlcodebook19.do
/*txt
//ho_ignore {pstd}See the code in {stata "doedit htmlcodebook19.do"}{p_end}

{pstd}
{cmd:Descvars} creates markers, and {cmd:Descvalues} adds links to those 
markers

{pstd}
that way users can now jump from the list of value labels back to the list of
variables.
txt*/

//codefile htmlcodebook20.do
/*txt

//ho_ignore {pstd}See the code in {stata "doedit htmlcodebook20.do"}{p_end}

{pstd}
Add some additional HTML code to make the tables look pretty. 
txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Turn this into an .ado file

/*txt
{pstd}
To turn this into an .ado file all we have to do is 

{pmore}
remove the commands outside the programs.

{pmore}
move the main program {cmd:htmlcodebook} to the top of the file

{pmore}
Add a comment {cmd:*! version 0.1.0 24Apr2018 MLB}, this is displayed when you
type {cmd:which htmlcodebook}

{pmore}
Store this under {cmd:htmlcodebook.ado}, the name of the file has to correspond
with the name of the first program

{pmore}
All other programs are local to the main program. So other programs cannot see them.
txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title What is still left?

/*txt
{pstd}
Some datasets consist of mulitple files, it would be nice to be able to specify
a directory and let htmlcodebook create a codebook for all files in that 
directory.

{pstd}
No program is complete without a help file. We can use {help examplehelpfile}
and {stata "viewsource examplehelpfile.sthlp"} as a template.

txt*/
//endslide ---------------------------------------------------------------------


